package planet.commonapi;

/**
 * @(#) NodeHandle.java
 *
 * This interface is an abstraction of a node handle from the CommonAPI paper. A
 * node handle is a handle to a known node, which conceptually includes the
 * node's Id, as well as the node's underlying network address (such as IP/port).
 *
 * @author Jordi Pujol 
 */
public abstract class NodeHandle  extends java.util.Observable implements java.io.Serializable {
	
	//constants defining types of observable events
	public static final Integer PROXIMITY_CHANGED = new Integer(1);
	public static final Integer DECLARED_DEAD = new Integer(2);
	public static final Integer DECLARED_LIVE = new Integer(3);
	
	/**
	 * Returns this node's id.
     *
     * @return The corresponding node's id.  
	 */
	public abstract Id getId();
	
	/**
	 * Returns whether or not this node is currently alive
	 *
	 * @return Whether or not this node is currently alive
	 */ 
	public abstract boolean isAlive();
	
	/**
	 * Updates the alive flag.
	 * @param alive New value for the alive flag.
	 */
	public abstract void setAlive(boolean alive);
	
	/**
	 * Evaluates the proximity between the node who invokes this method,
	 * and the node that represents this NodeHandle.
	 * @return The proximity between the two nodes, who invokes and 
	 * the node represented by this NodeHandle.
	 */
	public abstract int getProximity();
	
}
