package planet.commonapi.factory;

import planet.commonapi.*;
import planet.commonapi.exception.*;

/**
 * It's a factory for NodeHandles abstraction. It pretends build NodeHandles of any
 * class. It has got no parameters.
 * 
 * Theese two parameters are fixed by properties file
 * or by use of the NodeHandleFactory's methods. In the properties file exists the
 * following parameter's names:
 * <ul>
 * <li><b>DEFAULT_NODEHANDLECLASS</b>: It shows the class to use to generate new
 * NodeHandles.</li>
 * </ul>
 * IMPORTANT: The concrete implementation of NodeHandle must have one constructor:
 * <ol>
 * <li>The constructor with the related Node Id in its first parameter, and a 
 * boolean to show if the Node is alive, at the sencod argument.</li>
 * <ol>  
 * @author Jordi Pujol
 * @see planet.commonapi.NodeHandle
 */
public interface NodeHandleFactory extends java.io.Serializable {
	/**
	 * Builds a NodeHandle using the default NodeHandle class.
	 * @return A new instance of the default NodeHandle.
	 */
	public NodeHandle buildNodeHandle(Id nodeId, boolean alive) throws InitializationException;
	
	/**
	 * Gets Class for the generated NodeHandles.
	 * @return Class for the generated NodeHandles.
	 */
	public Class getNodeHandleClass();
	
	/**
	 * Sets Class to generate new NodeHandles.
	 * @param nodeClass Class for the NodeHandles to build in the future.
	 */
	public void setNodeHandleClass(Class nodeHandleClass) throws InitializationException;	
}
