package planet.generic.commonapi;

import planet.commonapi.exception.InitializationException;
import planet.generic.commonapi.factory.GenericFactory;
import planet.util.Properties;

/**
 * This class only attemps to help to programmers make more simple the initialization of the 
 * required context of new applications. Are two ways to initialize the context:
 * <br><br><b>One</b><br>
 * The new application must extends this GenericApp and includes in theirs constructors the
 * sentence <b>super()</b> or <b>super(factoriesProperties,simProperties,chordProperties)</b>.
 * For example:
 * <b><pre>
 * public NewTest() throws InitializationException {
 *     super(factoriesProperties,simProperties,chordProperties);
 * </pre></b>
 * where parameters of super() constructor are the path to specified files.
 * <br><br><b>Two</b><br>
 * Includes theese lines before anything that make reference to the network and the simulator and ...
 * <b><pre>
 * Properties.init(factoriesProperties,simProperties,chordProperties);
 * GenericFactory.init();
 * </pre></b>
 * where parameters of init() method are the path to specified files.
 * 
 * @author Jordi Pujol
 */
public class GenericApp {
	
	/**
	 * Make required initialization on the context of the application. In concrete:
	 * <b><pre>
	 * Properties.init("../conf/factories.properties","../conf/sim.properties","../conf/chord.properties");
  	 * GenericFactory.init();
  	 * </pre></b>
  	 * Because of theese lines, the new application must be executed under <b>bin</b> directory.
	 * 
	 * @throws InitializationException if any error has occurred during initialization of the context.
	 */
	public GenericApp() throws InitializationException {
  		Properties.init("../conf/factories.properties","../conf/sim.properties","../conf/chord.properties");
  		GenericFactory.init();
	}
	
	/**
	 * Make required initialization on the context of the application. In concrete:
	 * <b><pre>
	 * Properties.init(factoriesProperties,simProperties,chordProperties);
     * GenericFactory.init();
	 * </pre></b>
	 * Because of theese lines, the new application must be executed under the implicitly specified
	 * directory by the parameters files.
	 * @param factoriesProperties Path to properties file of factories attributes.
	 * @param simProperties Path to properties file of simulator attributes.
	 * @param chordProperties Path to properties file of chord attributes.
	 * @throws InitializationException if any error has occurred during initialization of the context.
	 */
	public GenericApp(String factoriesProperties,String simProperties, String chordProperties) throws InitializationException {
  		Properties.init(factoriesProperties,simProperties,chordProperties);
  		GenericFactory.init();
	}
}
