/*
 * Created on 15/04/2004
 *
 * To change the template for this generated file go to
 * Window - Preferences - Java - Code Generation - Code and Comments
 */
package planet.io;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;

/**
 * This class abstracts an stream that is capable of no replace 
 * the new saved status, according a specified boolean flag.
 * If the file must not be replaced, a new file is created with
 * a filename of type "filename_x", where "filename" is the 
 * specified base filename, and "x" is an iterative version number.
 * That is, if "out" filename just exists, it attempts to create
 * a new file "out_1"; if this just exist, it attempts to create
 * a new file named "out_2"; ... 
 * @author Jordi Pujol
 */
public class NoReplacingOutputStream extends OutputStream {
	/**
	 * Shows if the file must be replaced with the new status to save. 
	 */
	private boolean replace;
	/**
	 * Shows the base filename that must save the new status.
	 */
	private String filename = null;
	/**
	 * FileOutputStream stream to save the bytes stream.
	 */
	private FileOutputStream fos = null;
	
	/**
	 * Constructor that initialize this stream with the specified base
	 * <b>filename</b>. The <b>replace</b> flag shows if the file must
	 * be replaced with new save status or not. If must not be replaced,
	 * a suffix "_x" is added to the filename, where "x" is the number of 
	 * version.
	 * @param filename Base filename to save new status.
	 * @param replace Boolean flag that shows if the file must be replaced.
	 * @throws FileNotFoundException if the new desired file just exists.
	 */
	public NoReplacingOutputStream(String filename, boolean replace) throws FileNotFoundException {
		this.filename = filename;
		this.replace = replace;
		int version = 0;
		if (!replace) {
			File f = new File(this.filename);
			while (f.exists()) {
				version++;
				this.filename = this.filename + "_" + version;
			}
		}
		this.fos = new FileOutputStream(this.filename);
	}
	
	/**
	 * @see java.io.OutputStream#write(int)
	 */
	public void write(int b) throws IOException {
		fos.write(b);
	}
	
	/**
	 * @see java.io.OutputStream#close()
	 */
	public void close() throws IOException {
		fos.close();
	}
	
	/**
	 * @see java.io.OutputStream#flush()
	 */
	public void flush() throws IOException {
		fos.flush();
	}
	
	/**
	 * @see java.io.OutputStream#write(byte[], int, int)
	 */
	public void write(byte[] b, int off, int len) throws IOException {
		fos.write(b, off, len);
	}
	
	/**
	 * @see java.io.OutputStream#write(byte[])
	 */
	public void write(byte[] b) throws IOException {
		fos.write(b);
	}
}
