
package planet.simulate;

import planet.util.Properties;

/**
 * This class contains as static manner all of the types and
 * string representatations of them requireds for the simulator. 
 * @author Pedro Garca
 * @author Ruben Mondejar
 * @author Jordi Pujol 
 */
public class Globals {
	/**
	 * Maximum number of events to be processed by a simulator
	 */ 
	public static int ALARMS	=		100000;
	
	/* ******************  CONSTANTS FOR EVENTS *********************/
	
	/**
	 * For internal use to correct value for the TYPES array.
	 */
	private static final int MINIMUM	= 200;
	/**
	 * Event type: Adding a new node to the network.
	 */
	public final static int JOIN		= 200;
	/**
	 * Event type: Node leaving network.
	 */
	public final static int LEAVE		= 201;
	/**
	 * Event type: Node lookup.
	 */
	public final static int LOOKUP		= 202;
	/**
	 * Event type: Node fails.
	 */
	public final static int FAIL 		= 203;
	
	/* END ******************  CONSTANTS FOR EVENTS *********************/
	
	/* ******************  CONSTANTS FOR MODE OF ROUTEMESSAGE *******/
	/**
	 * Mode value: This mode is returned by the simulator always when the
	 * receiver of a message not exist in the actual network. 
	 */
	public final static int ERROR 		= 204;
	
	/* END ******************  CONSTANTS FOR MODE OF ROUTEMESSAGE *******/

	/**
	 * This String contains a string representation of each of 
	 * event types and RouteMessage type.
	 */	
	private final static String[] TYPES = {"JOIN","LEAVE","LOOKUP","FAIL","ERROR"}; 

	/**
	 * Returns a string representation of each of event type and 
	 * RouteMessage mode.
	 * @param type Type or mode of the RouteMessage to get its 
	 * String representation.
	 * @return String representation of the type or mode of RouteMessage.
	 */
	public static final String typeToString(int type) {
		try {
			return Properties.overlayProps.typeToString(type);
		} catch(ArrayIndexOutOfBoundsException e) {
			return TYPES[type-MINIMUM];
		}
	}
}
