package planet.test;

import java.util.Vector;

import planet.commonapi.Network;
import planet.generic.commonapi.factory.GenericFactory;
import planet.simulate.EventParser;
import planet.simulate.NetworkSimulator;
import planet.simulate.Scheduler;
import planet.util.Properties;
/**
 * This test use the event file "test_join1000d.txt" in the "bin" directory. Shows the time elapsed
 * for generating the network and its stabilization. Finally, prints all nodes with theirs
 * predecessors, finger table, and successor lists.
 * @author Ruben Mondejar
 * @author Jordi Pujol
 */
 
public class SimTest {

	/**
	 * Initialize the planet.util.Properties with 
	 * ../conf/factories_zero.properties, ../conf/sim.properties, ../conf/chord.properties files,
	 * with a test_join1000d.txt for event file.
	 * Then, it initialize the GenericFactory and starts the stabilization process.
	 * @param args Nothing.
	 * @throws Exception
	 */
	public static void main(String[] args) throws Exception {
		//init context
		Properties.init("../conf/factories_zero.properties","../conf/sim_simtest.properties","../conf/chord.properties");
		GenericFactory.init();
		
		//load events
		Vector events = EventParser.parseEvents(Properties.eventFile);
		Scheduler timer = new Scheduler();
		timer.addEvents(events);
			
		long t1,t2,t3,t4;
		
		//actual millis
		t1 = System.currentTimeMillis();
		NetworkSimulator sim = new NetworkSimulator(timer);
		t2 = System.currentTimeMillis();
		Network net = sim.getInternalNetwork();
		int steps = net.getSimulatedSteps();
		System.out.println("Network creation time ["+((t2-t1)/1000)+"."+((t2-t1)%1000)+"] seconds for ["+net.size()+"] nodes with ["+steps+"] steps.");
		
		//stabilization
		t3 = System.currentTimeMillis();
		sim.stabilize();
		t4 = System.currentTimeMillis();
		int steps2 = net.getSimulatedSteps();
		System.out.println("Simulation time ["+((t4-t3)/1000)+"."+((t4-t3)%1000)+"] seconds for ["+net.size()+"] nodes with ["+(steps2-steps)+"] steps.");
		System.out.println("Total time ["+((t4-t1)/1000)+"."+((t4-t1)%1000)+"] seconds for ["+net.size()+"] nodes with ["+steps2+"] steps.");
		sim.printNodes();
		}
}
