package planet.test.bad;


import planet.commonapi.Network;
import planet.commonapi.factory.NodeFactory;
import planet.generic.commonapi.factory.GenericFactory;
import planet.util.Properties;
/**
 * This test use the event file "test_join1000d.txt" in the "bin" directory. Shows the time elapsed
 * for generating the network and its stabilization. Finally, prints all nodes with theirs
 * predecessors, finger table, and successor lists.
 * @author Ruben Mondejar
 * @author Jordi Pujol
 */
 
public class SimNetTest {

	/**
	 * Initialize the planet.util.Properties with 
	 * ../conf/factories_zero.properties, ../conf/sim.properties, ../conf/chord.properties files,
	 * with a test_join1000d.txt for event file.
	 * Then, it initialize the GenericFactory and starts the stabilization process.
	 * @param args Nothing.
	 * @throws Exception
	 */
	public static void main(String[] args) throws Exception {
		//init context
		int max = 98;
		Properties.init("../conf/factories_zero.properties","../conf/sim_simtest.properties","../conf/chord.properties");
		GenericFactory.init();
		NodeFactory nf = GenericFactory.getNodeFactory();
		nf.setNodeClass(Class.forName("planet.badchord.BadChordNode"));
		
		//load events
		long t1,t2,t3,t4;
		
		//actual millis
		t1 = System.currentTimeMillis();
		Network net = GenericFactory.buildNetwork();
		for (int i=0; i<10;i++) {
			net.joinNodes(max); //add 98 ChordNode
			net.joinNode(nf.buildNode()); //add 2 BadChordNode
			net.joinNode(nf.buildNode());
		}
		t2 = System.currentTimeMillis();
		int steps = net.getSimulatedSteps();
		System.out.println("Network creation time ["+((t2-t1)/net.size())+"."+((t2-t1)%net.size())+"] seconds for ["+net.size()+"] nodes with ["+steps+"] steps.");
		
		//stabilization
		t3 = System.currentTimeMillis();
		int steps2 = net.stabilize();
		t4 = System.currentTimeMillis();
		//steps = net.getSimulatedSteps();
		System.out.println("Simulation time ["+((t4-t3)/net.size())+"."+((t4-t3)%net.size())+"] seconds for ["+net.size()+"] nodes with ["+(steps2-steps)+"] steps.");
		System.out.println("Total time ["+((t4-t1)/net.size())+"."+((t4-t1)%net.size())+"] seconds for ["+net.size()+"] nodes with ["+steps2+"] steps.");
		net.printNodes();
		}
}
