package planet.test.broadcast;

import planet.commonapi.Network;
import planet.commonapi.exception.InitializationException;
import planet.generic.commonapi.GenericApp;
import planet.generic.commonapi.factory.*;

/**
 * This test make a broadcast message over all network nodes.
 * @author Carles Pairot   <cpairot@etse.urv.es>
 * @author Jordi Pujol     <jordi.pujol@estudiants.urv.es>
 * @version 1.0
 */
public class BroadcastTest extends GenericApp {
  /**
   * Network to construct with all specified nodes at properties file.
   */	
  private static Network network = null;

  /**
   *  Constructor: inicialitza un node Chord
   *
   */
  public BroadcastTest() throws InitializationException {
	super("../conf/factories_broadcast.properties","../conf/sim.properties","../conf/chord.properties");
  	try {
  		//builds a network with the properties specified at properties file
  		long t1 = System.currentTimeMillis();
  		network = GenericFactory.buildNetwork();
  		int steps = network.stabilize();
  		long t2 = System.currentTimeMillis();
  		network.prettyPrintNodes();
  		System.out.println("Network created with ["+((t2-t1)/1000)+"."+((t2-t1)%1000)+"] seconds with ["+steps+"] steps\n");
  		//register to all nodes the application specified at properties files
  		network.registerApplicationAll();
  		System.out.println("Application added\n");
  		
  		} catch (InitializationException e) {
  			System.out.println("Occur an exception in initialization of "+this.getClass().getName()+": "+e.getMessage());
  			System.exit(-1);
  		}
  }

  /**
   * Per enviar un missatge
   */
  public void sendMessage() {
    System.out.println ("Sending message...");
    String messageText = "Hello World!";
    DHTApplication app = (DHTApplication)network.getRandomApplication(DHTApplication.applicationId);
    DHTPeerTestMessage mesg = new DHTPeerTestMessage (null,messageText);
    long t1 = System.currentTimeMillis();
    app.send(messageText,mesg);
    int steps = network.stabilize();
    long t2 = System.currentTimeMillis();
    System.out.println("\n\nBroadcast message delivered with ["+((t2-t1)/1000)+"."+((t2-t1)%1000)+"] seconds with ["+steps+"] steps\n");
	//network.printNodes();	
  }

  
  /**
   * Metode principal: rep els arguments i inicialitza el peer servidor
   * @param args
   */
  public static void main(String args[]) {

    try {
      BroadcastTest c = new BroadcastTest();
      // Enviem un missatge
      c.sendMessage();
    } catch (Exception e) {e.printStackTrace();}

    System.exit(0);
  }
}
