package planet.util;

import planet.commonapi.exception.InitializationException;

/**
 * This interface permits to implement as static form
 * This interficie allows to implement all the own properties of configuration
 * of the overlay network under a same instance.  It establishes a minimum 
 * contract, the method init(), so that their values are initialized. For this
 * reason, cast will have to be made in each place where it is necessary, 
 * to the concrete implementation.
 * @author <a href="mailto: jordi.pujol@estudiants.urv.es">Jordi Pujol</a>
 * Date: 05/07/2004
 */
public interface OverlayProperties extends java.io.Serializable {
	
	/**
	 * Initialize correctly the overlay configuration properties.
	 * @throws InitializationException if an error occurs during
	 * the initialization of the different properties.
	 * @param propertiesFile File with all configuration properties
	 * of the concrete overlay. This file must to follow the 
	 * java.util.Properties contract.
	 */
	public void init(String propertiesFile) throws InitializationException;
	
	/**
	 * Returns a String representation of the constant specific values
	 * of type and mode of the RouteMessage. Its use is only for
	 * human readable logs.
	 * @param type Value to get its String representation.
	 * @return The String representation of the type.
	 */
	public String typeToString(int type);
}
