package planet.util;

import java.io.*;

import planet.commonapi.exception.InitializationException;
import planet.generic.commonapi.factory.Topology;

/**
 * 
 * @author Jordi Pujol
 */
public class Properties {
	
	/* ************************** FILES OF PROPERTIES ********************/
	/**
	 * Default file which contains the default properties for the factories.
	 */
	public static final String DEFAULT_FACTORY_PROPERTIES = "../conf/factories.properties";
	/**
	 * Default file which contains the default properties for the simulator.
	 */
	public static final String DEFAULT_SIM_PROPERTIES = "../conf/sim.properties";
	/**
	 * Default file which contains the defualt properties for the Chord protocol.
	 */
	public static final String DEFAULT_CHORD_PROPERTIES = "../conf/chord.properties";
	
	/* ************************** PROPERTIES FOR FACTORIES ********************/
	/**
	 * Factories property: The default key specified in the properties file that
	 * identifies the class of the network implementation to use.
	 */
	public static final String DEFAULT_NETWORK = "DEFAULT_NETWORK";
	
	/**
	 * Factories property: The default key specified in the properties file that identifies
	 * the class of some Application's implementation.
	 */
	public static final String DEFAULT_APPLICATION = "DEFAULT_APPLICATION";
	/**
	 * Factories property: The default Class for Ids specified in the properties file.
	 */
	public static final String DEFAULT_ID_CLASS = "DEFAULT_IDCLASS";
	/**
	 * Factories property: The default IdFactory key specified in the properties file.
	 */
	public static final String DEFAULT_IDFACTORY = "DEFAULT_IDFACTORY";
	/**
	 * Factories property: The default Class for Nodes specified in the properties file.
	 */
	public static final String DEFAULT_NODECLASS = "DEFAULT_NODECLASS";
	/**
	 * Factories property: The default Class for NodeHandles specified in the properties file.
	 */
	public static final String DEFAULT_NODEHANDLECLASS = "DEFAULT_NODEHANDLECLASS";
	/**
	 * Factories property: Default key at properties file that identifies the NodeFactory.
	 */
	public static final String DEFAULT_NODE_FACTORY    = "DEFAULT_NODEFACTORY";
	/**
	 * Factories property: Default key at properties file that identifies the network size.
	 */
	public static final String DEFAULT_NETWORK_SIZE    = "DEFAULT_SIZE";
	/**
	 * Factories property: Default key at properties file that identifies the number of bits of the key.
	 */
	public static final String DEFAULT_BITS_KEY        = "DEFAULT_K";
	/**
	 * Factories property: Default key at properties file that identifies the serialized file that
	 * contains the state to load.
	 */
	public static final String DEFAULT_SERIALIZED_FILE = "DEFAULT_SERIALIZEDFILE";
	/**
	 * Factories property: Default key at properties file that identifies the topology of the network.
	 */
	public static final String DEFAULT_TOPOLOGY = "DEFAULT_TOPOLOGY";
	/**
	 * Factories property: Default key at properties file that identifies the class for new RouteMessage. 
	 */
	public static final String DEFAULT_ROUTE_MESSAGE = "DEFAULT_ROUTEMESSAGE";
	/**
	 * Factories property: Default key at properties file that identifies the class for new EndPoints.
	 */
	public static final String DEFAULT_END_POINT = "DEFAULT_ENDPOINT";
	
	/* *********************** SIMULATOR PROPERTIES ***********************/
	/**
	 * Simulator property: Default key for filename with events to load.
	 */
	public static final String DEFAULT_EVENT_FILE = "DEFAULT_EVENT_FILE";
	/**
	 * Simulator property: Default key for number of stabilization steps for any node join or leave.
	 */
	public static final String DEFAULT_SIMULATION_STEPS = "DEFAULT_SIMULATION_STEPS";
	/**
	 * Simulator property: Default key for log level.
	 */
	public static final String DEFAULT_LOG_LEVEL = "DEFAULT_LOG_LEVEL";
	/**
	 * Simulator property: Default key for filename to which serialize the final state.
	 */
	public static final String DEFAULT_OUTPUT_FILE = "DEFAULT_OUTPUT_FILE";
	/**
	 * Simulator property: Default key to show if the output file must be replaced with new outputs.
	 */
	public static final String DEFAULT_REPLACE_OUTPUT_FILE = "DEFAULT_REPLACE_OUTPUT_FILE";
	/**
	 * Simulator property: Default key for Timer class to use at nodes.
	 */
	public static final String DEFAULT_TIMER = "DEFAULT_TIMER";
	/**
	 * Simulator property: Default key for the overlay properties class.
	 */
	public static final String DEFAULT_OVERLAY_PROPERTIES = "DEFAULT_OVERLAY_PROPERTIES";
	/**
	 * Chord property: Default key for queue size.
	 */
	public static final String DEFAULT_QUEUE_SIZE = "DEFAULT_QUEUE_SIZE";
	/**
	 * Chord property: Default key for message processig.
	 */
	public static final String DEFAULT_MSG_PROCESS = "DEFAULT_MSG_PROCESS";

	/* *********************** OVERLAY PROPERTIES ***********************/
	/**
	 * Chord property: Default key for queue size.
	 */
	//public static final String DEFAULT_QUEUE_SIZE = "DEFAULT_QUEUE_SIZE";
	/**
	 * Chord property: Default key for message processig.
	 */
	//public static final String DEFAULT_MSG_PROCESS = "DEFAULT_MSG_PROCESS";
	/**
	 * Chord property: Default key for number of stabilize steps.
	 */
	//public static final String DEFAULT_STABILIZE_STEPS = "DEFAULT_STABILIZE_STEPS";
	/**
	 * Chord property: Default key for number of steps to fix finger tables.
	 */
	//public static final String DEFAULT_FIX_FINGER_STEPS = "DEFAULT_FIX_FINGER_STEPS";
	/**
	 * Chord property: Default key for size of successor list.
	 */
	//public static final String DEFAULT_SUCC_LIST_MAX = "DEFAULT_SUCC_LIST_MAX";

	
	/* *************** ATTRIBUTES FOR FACTORIES  ****************/
	/**
	 * Class of Network implementation.
	 */
	public static Class networkClass;
	/**
	 * Class of Application implementation.
	 */
	public static Class appClass;
	/**
	 * Topology of the target network.
	 */
	public static String topology;
	/**
	 * Id class.
	 */
	public static Class idClass;
	/**
	 * Network size to build.
	 */
	public static int networkSize;
	/**
	 * NodeFactory implementation class.
	 */
	public static Class nodeFactClass;
	/**
	 * IdFactory implementation class.
	 */
	public static Class idFactory;
	/**
	 * Node implementation class.
	 */
	public static Class nodeClass;
	/**
	 * NodeHandle implementation class.
	 */
	public static Class nodeHandleClass;
	/**
	 * Number of bits for Id.
	 */
	public static int bitsKey;
	/**
	 * Filename to use to load some serialized state.
	 */
	public static String serializedFile;
	/**
	 * Class that implements RouteMessage interface.
	 */
	public static Class routeMessage;
	/**
	 * Implementation of  EndPoint interface.
	 */
	public static Class endPoint;
	
	/* *************** ATTRIBUTES FOR SIMULATOR  ****************/
	/**
	 * Filename to use to load events to the simulator.
	 */
	public static String eventFile;
	/**
	 * Number of steps to be runned after any node join or leave.
	 */
	public static int simulationSteps;
	/**
	 * Log level during simulation..
	 */
	public static int logLevel;
	/**
	 * Filename to use to save the actual state by serialization.
	 */
	public static String outputFile; 
	/**
	 * Flag to show if the output file must be replaced or not.
	 */
	public static boolean replaceOutputFile;
	/**
	 * Class to use for Timers at nodes.
	 */
	public static Class timerClass;
	/**
	 * Class with all overlay properties.
	 */
	public static Class overlayPropertiesClass;
	/**
	 * Size of message queues.
	 */
	public static int queueSize;
	/**
	 * Message processing.
	 */
	public static int msgProcess;

	/* *************** ATTRIBUTES FOR OVERLAY  ****************/
	
	/**
	 * This contains all properties for the concrete overlay.
	 */
	public static OverlayProperties overlayProps;
	/**
	 * Size of message queues.
	 */
	//public static int queueSize;
	/**
	 * Message processing.
	 */
	//public static int msgProcess;
	/**
	 * Number of steps for statibilization.
	 */
	//public static int stabilizeSteps;
	/**
	 * Number of steps to fix fingers.
	 */
	//public static int fixFingerSteps;
	/**
	 * Max size for successor list.
	 */
	//public static int succListMax;
	
	/**
	 * Load all properties for factories, simulator and chord with default files.
	 * @throws InitializationException if some error occur during initialization.
	 */
	/*public static void init() throws InitializationException {
		initFactories(DEFAULT_FACTORY_PROPERTIES);
		initSimulator(DEFAULT_SIM_PROPERTIES);
		//initOverlay(DEFAULT_CHORD_PROPERTIES);		
	}*/
	
	/**
	 * Load all properties for factories, simulator and chord, from the
	 * specified files, respectively.
	 * @param factoryProperties File that contains the factories properties.
	 * @param simProperties File that contains the simulator properties. 
	 * @param overlayProperties  File that contains the overlay properties.
	 * @throws InitializationException if some error occur during initialization.
	 */
	public static void init(String factoryProperties,String simProperties,String overlayProperties)
			throws InitializationException {
		initFactories(factoryProperties);
		initSimulator(simProperties);
		initOverlay(overlayProperties);		
	}
	
	/**
	 * Initializa all attributes for factories.
	 * @param filename File which contains all properties for the factories.
	 * @throws InitializationException if occur some error during initialization.
	 */
	public static void initFactories(String filename) throws InitializationException {
		//Load properties
		FileInputStream fis = null;
		String factoriesFault = "";
		java.util.Properties prop = new java.util.Properties(); 
		try {
			fis = new FileInputStream(filename);
			prop.load(fis);
		} catch (FileNotFoundException e) {
			throw new InitializationException("Cannot read the file "+filename,e);
		} catch (IOException e) {
			throw new InitializationException("Cannot load the file "+filename,e);
		}
		
		//if all OK
		
		//read defaultNetwork
		try {
			networkClass = Class.forName(prop.getProperty(DEFAULT_NETWORK));
		} catch (ClassNotFoundException e) {
			throw new InitializationException("Property ["+DEFAULT_NETWORK +"] is not a valid class name.",e);
		} 
		
		
		//read defaultApplication
		try {
			appClass = Class.forName(prop.getProperty(DEFAULT_APPLICATION));
		} catch (ClassNotFoundException e) {
			throw new InitializationException("Property ["+DEFAULT_APPLICATION +"] is not a valid class name.",e);
		} 
		
		//read defaultIdClass
		try {
			idClass = Class.forName(prop.getProperty(DEFAULT_ID_CLASS));
		} catch (ClassNotFoundException e) {
			throw new InitializationException("Property ["+DEFAULT_ID_CLASS +"] is not a valid class name.",e);
		} 
		
		//read defaultIdFactory
		try {
			idFactory = Class.forName(prop.getProperty(DEFAULT_IDFACTORY));
		} catch (ClassNotFoundException e) {
			throw new InitializationException("Property ["+DEFAULT_IDFACTORY +"] is not a valid class name.",e);
		}

		//read defaultNodeClass
		try {
			nodeClass = Class.forName(prop.getProperty(DEFAULT_NODECLASS));
		} catch (ClassNotFoundException e) {
			throw new InitializationException("Property ["+DEFAULT_NODECLASS +"] is not a valid class name.",e);
		}
		
		//read defaultNodeFactory
		try {
			nodeFactClass = Class.forName(prop.getProperty(DEFAULT_NODE_FACTORY));
		} catch (ClassNotFoundException e) {
			throw new InitializationException("Property "+DEFAULT_NODE_FACTORY +" is not a valid class name ["+ 
					prop.getProperty(DEFAULT_NODE_FACTORY) +"].",e);
		} 
		
		//read DEFAULT_NODEHANDLECLASS
		try {
			nodeHandleClass = Class.forName(prop.getProperty(DEFAULT_NODEHANDLECLASS));
		} catch (ClassNotFoundException e) {
			throw new InitializationException("Property ["+DEFAULT_NODECLASS +"] is not a valid class name.",e);
		}
		
		//read defaultTopology of network
		topology = prop.getProperty(DEFAULT_TOPOLOGY);
		if (!Topology.isValid(topology))
			throw new InitializationException("The topology ["+topology+"] is not a valid topology.");
		
		//read the defaultSize of network.
		networkSize = Integer.parseInt(prop.getProperty(DEFAULT_NETWORK_SIZE));
		if (networkSize < 0) {
			throw new InitializationException("The network size ["+networkSize+"] are invalid.");
		}
		
		//read defaultBitskey
		bitsKey = Integer.parseInt(prop.getProperty(DEFAULT_BITS_KEY));
		if (bitsKey > 160 || bitsKey%32!=0) {
			throw new InitializationException("The property ["+DEFAULT_BITS_KEY+"] has an invalid value. "
				+ "The only correct value is 160 or any 32*X in 32..160 range.");
		}
		
		//read defaultSerializedFile
		serializedFile = prop.getProperty(DEFAULT_SERIALIZED_FILE);
		
		//read defaultRouteMessage
		try {
			routeMessage = Class.forName(prop.getProperty(DEFAULT_ROUTE_MESSAGE));
		} catch (ClassNotFoundException e) {
			throw new InitializationException("Property ["+DEFAULT_ROUTE_MESSAGE +"] is not a valid class name.",e);
		}

		//read DEFAULT_END_POINT
		try {
			endPoint = Class.forName(prop.getProperty(DEFAULT_END_POINT));
		} catch (ClassNotFoundException e) {
			throw new InitializationException("Property ["+DEFAULT_END_POINT +"] is not a valid class name.",e);
		}
		
	}	
	
	/**
	 * Load all properties for the simulator.
	 * @param filename File that contains all properties for simulator.
	 * @throws InitializationException if occur some error during initialization.
	 */
	public static void initSimulator(String filename) throws InitializationException {
		//Load properties
		FileInputStream fis = null;
		java.util.Properties prop = new java.util.Properties();
		try {
			fis = new FileInputStream(filename);
			prop.load(fis);
		} catch (FileNotFoundException e) {
			throw new InitializationException("Cannot read the file "+filename,e);
		} catch (IOException e) {
			throw new InitializationException("Cannot load the file "+filename,e);
		}
		
		//if all OK

		//read the defaultEventFile
		eventFile = prop.getProperty(DEFAULT_EVENT_FILE);

		//read the defaultStabilizationSteps
		simulationSteps = Integer.parseInt(prop.getProperty(DEFAULT_SIMULATION_STEPS));
		if (simulationSteps < 0) 
			throw new InitializationException("Property ["+DEFAULT_SIMULATION_STEPS +"] is not a valid number of steps.");
		
		//read the defaultLogLevel number
		logLevel = Integer.parseInt(prop.getProperty(DEFAULT_LOG_LEVEL));
		if (logLevel < 0 || logLevel > 3) 
			throw new InitializationException("Property ["+DEFAULT_LOG_LEVEL +"] is not a valid log level.");
		
		//read the defualtOutputFile 
		outputFile = prop.getProperty(DEFAULT_OUTPUT_FILE);
		
		//read the defaultReplaceOutputFile
		replaceOutputFile = Boolean.getBoolean(prop.getProperty(DEFAULT_REPLACE_OUTPUT_FILE));
		
		//read DEFAULT_TIMER
		try {
			timerClass = Class.forName(prop.getProperty(DEFAULT_TIMER));
		} catch (ClassNotFoundException e) {
			throw new InitializationException("Property ["+DEFAULT_TIMER +"] is not a valid class name.",e);
		}
		
		//read DEFAULT_OVERLAY_PROPERTIES
		try {
			overlayPropertiesClass = Class.forName(prop.getProperty(DEFAULT_OVERLAY_PROPERTIES));
		} catch (ClassNotFoundException e) {
			throw new InitializationException("Property ["+DEFAULT_OVERLAY_PROPERTIES +"] is not a valid class name.",e);
		}
		
		//read the defaultQueueSize
		queueSize = Integer.parseInt(prop.getProperty(DEFAULT_QUEUE_SIZE));
		if (queueSize < 0) 
			throw new InitializationException("Property ["+DEFAULT_QUEUE_SIZE +"] is not a valid number of queue size.");
		
		//read the defaultMsgProcess
		msgProcess = Integer.parseInt(prop.getProperty(DEFAULT_MSG_PROCESS));
		if (msgProcess < 0) 
			throw new InitializationException("Property ["+DEFAULT_MSG_PROCESS +"] is not a valid number of message process.");

	}
	
	/**
	 * Initialize the overlay properties.
	 * @param overlayProperties Filename with all the configuration properties for
	 * the concrete overlay.
	 */
	private static void initOverlay(String overlayProperties) throws InitializationException {
		try {
			overlayProps = (OverlayProperties)overlayPropertiesClass.newInstance();
			overlayProps.init(overlayProperties);
		/*} catch (ClassNotFoundException e) {
			throw new InitializationException("Property ["+DEFAULT_OVERLAY_PROPERTIES +"] is not a valid class name.",e);*/
		} catch (InitializationException e) {
			throw e;
		} catch (Exception e) {
			throw new InitializationException("Cannot build a new instance with the overlay properties.",e);
		} 
	}
	
	/**
	 * Initialize all properties for Chord.
	 * @param filename File that contains the properties for Chord.
	 * @throws InitializationException if occur some error during initialization.
	 */
	/*public static void initChord(String filename) throws InitializationException {
		//Load properties
		FileInputStream fis = null;
		java.util.Properties prop = new java.util.Properties();
		try {
			fis = new FileInputStream(filename);
			prop.load(fis);
		} catch (FileNotFoundException e) {
			throw new InitializationException("Cannot read the file "+filename,e);
		} catch (IOException e) {
			throw new InitializationException("Cannot load the file "+filename,e);
		}
		
		//if all OK
	
		//read the defaultQueueSize
		queueSize = Integer.parseInt(prop.getProperty(DEFAULT_QUEUE_SIZE));
		if (queueSize < 0) 
			throw new InitializationException("Property ["+DEFAULT_QUEUE_SIZE +"] is not a valid number of queue size.");
		
		//read the defaultMsgProcess
		msgProcess = Integer.parseInt(prop.getProperty(DEFAULT_MSG_PROCESS));
		if (msgProcess < 0) 
			throw new InitializationException("Property ["+DEFAULT_MSG_PROCESS +"] is not a valid number of message process.");

		//read the defaultStabilizeSteps
		stabilizeSteps = Integer.parseInt(prop.getProperty(DEFAULT_STABILIZE_STEPS));
		if (stabilizeSteps < 0) 
			throw new InitializationException("Property ["+DEFAULT_STABILIZE_STEPS +"] is not a valid number of stabilize steps.");
		
		//read the defaultFixFingerSteps
		fixFingerSteps = Integer.parseInt(prop.getProperty(DEFAULT_FIX_FINGER_STEPS));
		if (fixFingerSteps < 0) 
			throw new InitializationException("Property ["+DEFAULT_FIX_FINGER_STEPS +"] is not a valid number of steps to fix finger tables.");

		//read the defaultSuccListMax
		succListMax = Integer.parseInt(prop.getProperty(DEFAULT_SUCC_LIST_MAX));
		if (succListMax < 0) 
			throw new InitializationException("Property ["+DEFAULT_SUCC_LIST_MAX +"] is not a valid size of successor list.");
	}	*/
}
