package planet.util;

import java.security.*;

/**
 * Offers different utilities.
 * @author Jordi Pujol
 */
public class Utilities {

	/**
	 * Change a byte array to int array
	 * @param data byte array to change to int array
	 * @return int array from data
	 */
	public static int[] toIntArray(byte[] data) {
		int[] newData = new int[data.length];
		for (int i=0; i<data.length;i++) {
			newData[i] = (int) data[i];
		}
		return newData;
	}
	
	/**
	 * Change a byte array to int array
	 * @param data byte array to change to int array
	 * @return int array from data
	 */
	public static int[] toIntArray(String data) {
		byte[] dataByte = data.getBytes();
		int[] newData = new int[dataByte.length];
		for (int i=0; i<dataByte.length;i++) {
			newData[i] = (int) dataByte[i];
		}
		return newData;
	}
	
	/**
	 * Generate a hash code from specified data. Hash function is SHA-1. 
	 * @param data String to obtain its hash code.
	 * @return null if no SHA could be obtained, or result of hash function in 
	 * int array format.
	 */
	public static int[] generateIntHash (String data) {
	  MessageDigest md = null;
	  byte[] digest = null;

	  try {
		md = MessageDigest.getInstance("SHA");
		md.update(data.getBytes());
		digest = md.digest();
	  }
	  catch (NoSuchAlgorithmException e) {
		System.err.println("No SHA support!");
	  }
	  
	  return toIntArray(digest);
	}

	/**
	 * Generate a hash code from specified data. Hash function is SHA-1. 
	 * @param data String to obtain its hash code.
	 * @return null if no SHA could be obtained, or result of hash function 
	 * in byte array format.
	 */
	public static byte[] generateByteHash (String data) {
	  MessageDigest md = null;
	  byte[] digest = null;

	  try {
		md = MessageDigest.getInstance("SHA");
		md.update(data.getBytes());
		digest = md.digest();
	  }
	  catch (NoSuchAlgorithmException e) {
		System.err.println("No SHA support!");
	  }
	  return digest;
	}


}
